#include <ishtar/network.h>
#include <iostream>
#include <fstream>
#include <cstdlib>

// Simple network client that dumps data from the network
class SimpleNetworkServer : public Ishtar::NetworkServer
{
protected:
	// Read a string terminated by '\n'
	void getString(std::string &s, Ishtar::Socket *socket)
	{
		Ishtar::UInt8 c;
		while ((c = socket->readUInt8()) != '\n')
			s += c;
	}
public:
	// Constructor, just create Ishtar::NetworkServer
	SimpleNetworkServer(Ishtar::UInt16 port) : Ishtar::NetworkServer(port) { }
	
	// Incoming data, print them
	virtual void incomingData (Ishtar::Socket *socket)
	{
		// Trivial HTTP header parser
		std::string request;
		getString(request, socket);
		if (request.substr(0, 3) == "GET")
		{
			// Get filename
			std::string::size_type n = request.find_first_of(' ', 4);
			std::string filename = request.substr(4, n - 4);
			
			// Open file
			std::ifstream file(filename.c_str(), std::ios::binary);
			
			// Get length of file
			file.seekg (0, std::ios::end);
			size_t length = file.tellg();
			file.seekg (0, std::ios::beg);
			
			// Read file and write to socket
			char *buffer = new char[length];
			file.read(buffer, length);
			file.close();
			socket->write(buffer, length);
			socket->flush();
			
			// Close socket
			socket->disconnect();
			delete[] buffer;
		}
		else
			std::cerr << "Unknown request " << request << std::endl;
	}
	
	// Incoming connection, print name of host
	virtual void incomingConnection (Ishtar::Socket *socket)
	{
		std::cout << "Host " << socket->getRemoteName() << " has closed connected." << std::endl;
	}
	
	// Remote host has closed, print its name
	virtual void connectionClosed (Ishtar::Socket *socket)
	{
		std::cout << "Host " << socket->getRemoteName() << " has closed connection." << std::endl;
	}
};

int main(int argc, char *argv[])
{
	// we want the hostname and port as argument
	if (argc != 2)
	{
		std::cerr << "Usage " << argv[0] << " port" << std::endl;
		exit(1);
	}
	
	// Create an instance of TCP/IP server
	SimpleNetworkServer server(atoi(argv[1]));
	
	// Run the server forever
	server.run();
}